@@ -35,6 +35,8 @@
  *     use xlockdisplay() or use two seperate connections to the x-server
  *
  */
+/*_x_ Lines formatted like this one are xine-lib developer comments.  */
+/*_x_ They will be removed from the installed version of this header. */
 
 #ifndef HAVE_XINE_H
 #define HAVE_XINE_H
@@ -53,13 +55,9 @@ extern "C" {
 #include <windowsx.h>
 #endif
 
-#ifdef XINE_COMPILE
-#include <inttypes.h>
-#include "attributes.h"
-#else
 #include <xine/os_types.h>
 #include <xine/attributes.h>
-#endif
+#include <xine/version.h>
 
 /* This enables some experimental features. These are not part of the
  * official libxine API, so use them only, if you absolutely need them.
@@ -85,13 +83,6 @@ typedef struct xine_stream_s xine_stream_t;
 typedef struct xine_audio_port_s xine_audio_port_t;
 typedef struct xine_video_port_s xine_video_port_t;
 
-#ifndef XINE_DISABLE_DEPRECATED_FEATURES
-/* convenience types: simple player UIs might want to call ports drivers */
-typedef xine_audio_port_t xine_ao_driver_t XINE_DEPRECATED;
-typedef xine_video_port_t xine_vo_driver_t XINE_DEPRECATED;
-#endif
-
-
 /*********************************************************************
  * global engine handling                                            *
  *********************************************************************/
@@ -108,12 +99,6 @@ void xine_get_version (int *major, int *minor, int *sub) XINE_PROTECTED;
    return 1 if compatible, 0 otherwise */
 int  xine_check_version (int major, int minor, int sub) XINE_PROTECTED;
 
-/* static info - which libxine release this header came from */
-#define XINE_MAJOR_VERSION 1
-#define XINE_MINOR_VERSION 1
-#define XINE_SUB_VERSION   20
-#define XINE_VERSION       "1.1.20.1"
-
 /*
  * pre-init the xine engine
  *
@@ -132,6 +117,12 @@ int  xine_check_version (int major, int minor, int sub) XINE_PROTECTED;
  */
 xine_t *xine_new (void) XINE_PROTECTED;
 
+/* allow the setting of some flags before xine_init
+ * FIXME-ABI: this is currently GLOBAL
+ */
+void xine_set_flags (xine_t *, int) XINE_PROTECTED XINE_WEAK;
+#define XINE_FLAG_NO_WRITE_CACHE		1
+
 /*
  * post_init the xine engine
  */
@@ -251,22 +242,6 @@ int xine_open (xine_stream_t *stream, const char *mrl) XINE_PROTECTED;
  */
 int  xine_play (xine_stream_t *stream, int start_pos, int start_time) XINE_PROTECTED;
 
-/*
- * set xine to a trick mode for fast forward, backwards playback,
- * low latency seeking. Please note that this works only with some
- * input plugins. mode constants see below.
- *
- * returns 1 if OK, 0 on error (use xine_get_error for details)
- */
-int  xine_trick_mode (xine_stream_t *stream, int mode, int value) XINE_PROTECTED XINE_DEPRECATED;
-
-/* trick modes */
-#define XINE_TRICK_MODE_OFF                0
-#define XINE_TRICK_MODE_SEEK_TO_POSITION   1
-#define XINE_TRICK_MODE_SEEK_TO_TIME       2
-#define XINE_TRICK_MODE_FAST_FORWARD       3
-#define XINE_TRICK_MODE_FAST_REWIND        4
-
 /*
  * stop stream playback
  * xine_stream_t stays valid for new xine_open or xine_play
@@ -296,7 +271,7 @@ void xine_dispose (xine_stream_t *stream) XINE_PROTECTED;
  */
 void xine_engine_set_param(xine_t *self, int param, int value) XINE_PROTECTED;
 int xine_engine_get_param(xine_t *self, int param) XINE_PROTECTED;
-  
+
 #define XINE_ENGINE_PARAM_VERBOSITY        1
 
 /*
@@ -338,7 +313,7 @@ int  xine_get_param (xine_stream_t *stream, int param) XINE_PROTECTED;
 #define XINE_PARAM_EQ_16000HZ             27 /* equalizer gains -100..100   */
 #define XINE_PARAM_AUDIO_CLOSE_DEVICE     28 /* force closing audio device  */
 #define XINE_PARAM_AUDIO_AMP_MUTE         29 /* 1=>mute, 0=>unmute */
-#define XINE_PARAM_FINE_SPEED             30 /* 1.000.000 => normal speed   */ 
+#define XINE_PARAM_FINE_SPEED             30 /* 1.000.000 => normal speed   */
 #define XINE_PARAM_EARLY_FINISHED_EVENT   31 /* send event when demux finish*/
 #define XINE_PARAM_GAPLESS_SWITCH         32 /* next stream only gapless swi*/
 #define XINE_PARAM_DELAY_FINISHED_EVENT   33 /* 1/10sec,0=>disable,-1=>forev*/
@@ -356,7 +331,7 @@ int  xine_get_param (xine_stream_t *stream, int param) XINE_PROTECTED;
 #define XINE_SPEED_NORMAL                  4
 #define XINE_SPEED_FAST_2                  8
 #define XINE_SPEED_FAST_4                  16
- 
+
 /* normal speed value for XINE_PARAM_FINE_SPEED parameter */
 #define XINE_FINE_SPEED_NORMAL             1000000
 
@@ -374,12 +349,13 @@ int  xine_get_param (xine_stream_t *stream, int param) XINE_PROTECTED;
 #define XINE_PARAM_VO_TVMODE               0x0100000a /* ???                */
 #define XINE_PARAM_VO_WINDOW_WIDTH         0x0100000f /* readonly           */
 #define XINE_PARAM_VO_WINDOW_HEIGHT        0x01000010 /* readonly           */
+#define XINE_PARAM_VO_SHARPNESS            0x01000018 /* 0..65535           */
+#define XINE_PARAM_VO_NOISE_REDUCTION      0x01000019 /* 0..65535           */
 #define XINE_PARAM_VO_CROP_LEFT            0x01000020 /* crop frame pixels  */
 #define XINE_PARAM_VO_CROP_RIGHT           0x01000021 /* crop frame pixels  */
 #define XINE_PARAM_VO_CROP_TOP             0x01000022 /* crop frame pixels  */
 #define XINE_PARAM_VO_CROP_BOTTOM          0x01000023 /* crop frame pixels  */
 
-
 #define XINE_VO_ZOOM_STEP                  100
 #define XINE_VO_ZOOM_MAX                   400
 #define XINE_VO_ZOOM_MIN                   -85
@@ -418,7 +394,7 @@ int  xine_get_param (xine_stream_t *stream, int param) XINE_PROTECTED;
  * image format can be YUV 4:2:0 or 4:2:2
  * will copy the image data into memory that <img> points to
  * (interleaved for yuv 4:2:2 or planary for 4:2:0)
- * 
+ *
  * xine_get_current_frame() requires that <img> must be able
  * to hold the image data. Use a NULL pointer to retrieve the
  * necessary parameters for calculating the buffer size. Be
@@ -432,7 +408,7 @@ int  xine_get_param (xine_stream_t *stream, int param) XINE_PROTECTED;
  * xine_get_current_frame_alloc() takes care of allocating
  * a buffer on its own, so image data can be retrieved by
  * a single call without the need to pause the stream.
- * 
+ *
  * xine_get_current_frame_data() passes the parameters of the
  * previously mentioned functions plus further information in
  * a structure and can work like the _s or _alloc function
@@ -481,6 +457,8 @@ int  xine_get_current_frame_data (xine_stream_t *stream,
 #define XINE_IMGFMT_YUY2 (('2'<<24)|('Y'<<16)|('U'<<8)|'Y')
 #define XINE_IMGFMT_XVMC (('C'<<24)|('M'<<16)|('v'<<8)|'X')
 #define XINE_IMGFMT_XXMC (('C'<<24)|('M'<<16)|('x'<<8)|'X')
+#define XINE_IMGFMT_VDPAU (('A'<<24)|('P'<<16)|('D'<<8)|'V')
+#define XINE_IMGFMT_VAAPI (('P'<<24)|('A'<<16)|('A'<<8)|'V')
 
 /* get current xine's virtual presentation timestamp (1/90000 sec)
  * note: this is mostly internal data.
@@ -489,6 +467,83 @@ int  xine_get_current_frame_data (xine_stream_t *stream,
 int64_t xine_get_current_vpts(xine_stream_t *stream) XINE_PROTECTED;
 
 
+/*
+ * Continuous video frame grabbing feature.
+ *
+ * In opposite to the 'xine_get_current_frame' based snapshot function this grabbing
+ * feature allow continuous grabbing of last or next displayed video frame.
+ * Grabbed video frames are returned in simple three byte RGB format.
+ *
+ * Depending on the capabilities of the used video output driver video image data is
+ * taken as close as possible at the end of the video processing chain. Thus a returned
+ * video image could contain the blended OSD data, is deinterlaced, cropped and scaled
+ * and video properties like hue, sat could be applied.
+ * If a video output driver does not have a decent grabbing implementation then there
+ * is a generic fallback feature that grabs the video frame as they are taken from the video
+ * display queue (like the xine_get_current_frame' function).
+ * In this case color correct conversation to a RGB image incorporating source cropping
+ * and scaling to the requested grab size is also supported.
+ *
+ * The caller must first request a new video grab frame using the public 'xine_new_grab_video_frame'
+ * function. Then the caller should populate the frame with the wanted source cropping, grab image
+ * size and control flags. After that grab requests could be done by calling the supplied grab() feature
+ * of the frame. At the end a call to the supplied dispose() feature of the frame releases all needed
+ * resources.
+ * The caller should have acquired a port ticket while calling these features.
+ *
+ */
+#define HAVE_XINE_GRAB_VIDEO_FRAME      1
+
+/*
+ * frame structure used for grabbing video frames of format RGB.
+ */
+typedef struct xine_grab_video_frame_s xine_grab_video_frame_t;
+struct xine_grab_video_frame_s {
+  /*
+   *  grab last/next displayed image.
+   *  returns 0 if grab is successful, 1 on timeout and -1 on error
+   */
+  int (*grab) (xine_grab_video_frame_t *self);
+
+  /*
+   *  free all resources.
+   */
+  void (*dispose) (xine_grab_video_frame_t *self);
+
+  /*
+   *  Cropping of source image. Has to be specified by caller.
+   */
+  int crop_left;
+  int crop_right;
+  int crop_top;
+  int crop_bottom;
+
+  /*
+   * Parameters of returned RGB image.
+   * Caller can specify wanted frame size giving width and/or height a value > 0.
+   * In this case the grabbed image is scaled to the requested size.
+   * Otherwise the grab function returns the actual size of the grabbed image
+   * in width/height without scaling the image.
+   */
+  int width, height; /* requested/returned size of image */
+  uint8_t *img;      /* returned RGB image data taking three bytes per pixel */
+  int64_t vpts;      /* virtual presentation timestamp (1/90000 sec) of returned frame */
+
+  int timeout;       /* Max. time to wait for next displayed frame in milliseconds */
+  int flags;         /* Controlling flags. See XINE_GRAB_VIDEO_FRAME_FLAGS_* definitions */
+};
+
+#define XINE_GRAB_VIDEO_FRAME_FLAGS_CONTINUOUS  0x01    /* optimize resource allocation for continuous frame grabbing */
+#define XINE_GRAB_VIDEO_FRAME_FLAGS_WAIT_NEXT   0x02    /* wait for next display frame instead of using last displayed frame */
+
+#define XINE_GRAB_VIDEO_FRAME_DEFAULT_TIMEOUT   500
+
+/*
+ * Allocate new grab video frame. Returns NULL on error.
+ */
+xine_grab_video_frame_t*  xine_new_grab_video_frame (xine_stream_t *stream) XINE_PROTECTED;
+
+
 /*********************************************************************
  * media processing                                                  *
  *********************************************************************/
@@ -514,17 +569,17 @@ typedef struct {
 
   int64_t  vpts;       /* timestamp 1/90000 sec for a/v sync */
   int64_t  duration;
+  double   aspect_ratio;
   int      width, height;
   int      colorspace; /* XINE_IMGFMT_* */
-  double   aspect_ratio;
 
   int      pos_stream; /* bytes from stream start */
   int      pos_time;   /* milliseconds */
 
+  int      frame_number; /* frame number (may be unknown) */
+
   uint8_t *data;
   void    *xine_frame; /* used internally by xine engine */
-  
-  int      frame_number; /* frame number (may be unknown) */
 } xine_video_frame_t;
 
 int xine_get_next_video_frame (xine_video_port_t *port,
@@ -542,11 +597,11 @@ typedef struct {
   int      num_channels;
   int      bits_per_sample; /* per channel */
 
-  off_t    pos_stream; /* bytes from stream start */
-  int      pos_time;   /* milliseconds */
-
   uint8_t *data;
   void    *xine_frame; /* used internally by xine engine */
+
+  off_t    pos_stream; /* bytes from stream start */
+  int      pos_time;   /* milliseconds */
 } xine_audio_frame_t;
 
 int xine_get_next_audio_frame (xine_audio_port_t *port,
@@ -554,20 +609,6 @@ int xine_get_next_audio_frame (xine_audio_port_t *port,
 
 void xine_free_audio_frame (xine_audio_port_t *port, xine_audio_frame_t *frame) XINE_PROTECTED;
 
-  /*
-   * maybe future aproach:
-   */
-
-int xine_get_video_frame (xine_stream_t *stream,
-			  int timestamp, /* msec */
-			  int *width, int *height,
-			  int *ratio_code, 
-			  int *duration, /* msec */
-			  int *format,
-			  uint8_t *img) XINE_PROTECTED XINE_DEPRECATED;
-
-/* TODO: xine_get_audio_frame */
-
 #endif
 
 
@@ -592,18 +633,18 @@ struct xine_post_s {
    * pass them to the initialization of streams
    */
   xine_audio_port_t **audio_input;
-  
+
   /* a NULL-terminated array of video input ports this post plugin
    * provides; you can hand these to other post plugin's outputs or
    * pass them to the initialization of streams
    */
   xine_video_port_t **video_input;
-  
+
   /* the type of the post plugin
    * one of XINE_POST_TYPE_* can be used here
    */
   int type;
-  
+
 };
 
 /*
@@ -625,7 +666,7 @@ xine_post_t *xine_post_init(xine_t *xine, const char *name,
 const char *const *xine_list_post_plugins(xine_t *xine) XINE_PROTECTED;
 
 /* get a list of all post plugins of one type */
-const char *const *xine_list_post_plugins_typed(xine_t *xine, int type) XINE_PROTECTED;
+const char *const *xine_list_post_plugins_typed(xine_t *xine, uint32_t type) XINE_PROTECTED;
 
 /*
  * post plugin input/output
@@ -643,28 +684,25 @@ struct xine_post_in_s {
 
   /* the name identifying this input */
   const char   *name;
-  
-  /* the datatype of this input, use one of XINE_POST_DATA_* here */
-  int           type;
-  
+
   /* the data pointer; input is directed to this memory location,
    * so you simply access the pointer to access the input data */
   void         *data;
-  
+
+  /* the datatype of this input, use one of XINE_POST_DATA_* here */
+  int           type;
+
 };
 
 struct xine_post_out_s {
 
   /* the name identifying this output */
   const char   *name;
-  
-  /* the datatype of this output, use one of XINE_POST_DATA_* here */
-  int           type;
-  
+
   /* the data pointer; output should be directed to this memory location,
    * so in the easy case you simply write through the pointer */
   void         *data;
-  
+
   /* this function is called, when the output should be redirected
    * to another input, you sould set the data pointer to direct
    * any output to this new input;
@@ -676,6 +714,9 @@ struct xine_post_out_s {
    * if you do not implement rewiring, set this to NULL */
   int (*rewire) (xine_post_out_t *self, void *data);
 
+  /* the datatype of this output, use one of XINE_POST_DATA_* here */
+  int           type;
+
 };
 
 /* get a list of all inputs of a post plugin */
@@ -774,14 +815,14 @@ void xine_post_dispose(xine_t *xine, xine_post_t *self) XINE_PROTECTED;
 /* defines a single parameter entry. */
 typedef struct {
   int              type;             /* POST_PARAM_TYPE_xxx             */
-  char            *name;             /* name of this parameter          */
+  const char      *name;             /* name of this parameter          */
   int              size;             /* sizeof(parameter)               */
   int              offset;           /* offset in bytes from struct ptr */
   char           **enum_values;      /* enumeration (first=0) or NULL   */
   double           range_min;        /* minimum value                   */
   double           range_max;        /* maximum value                   */
   int              readonly;         /* 0 = read/write, 1=read-only     */
-  char            *description;      /* user-friendly description       */
+  const char      *description;      /* user-friendly description       */
 } xine_post_api_parameter_t;
 
 /* description of parameters struct (params). */
@@ -893,6 +934,7 @@ int xine_get_audio_lang (xine_stream_t *stream, int channel,
 			 char *lang) XINE_PROTECTED;
 int xine_get_spu_lang   (xine_stream_t *stream, int channel,
 			 char *lang) XINE_PROTECTED;
+/*_x_ increasing this number means an incompatible ABI breakage! */
 #define XINE_LANG_MAX                     32
 
 /*
@@ -1026,8 +1068,8 @@ typedef struct {
   char      *origin; /* file plugin: path */
   char      *mrl;    /* <type>://<location> */
   char      *link;
-  uint32_t   type;   /* see below */
   off_t      size;   /* size of this source, may be 0 */
+  uint32_t   type;   /* see below */
 } xine_mrl_t;
 
 /* mrl types */
@@ -1070,9 +1112,9 @@ xine_mrl_t **xine_get_browse_mrls (xine_t *self,
 const char *const *xine_get_autoplay_input_plugin_ids (xine_t *self) XINE_PROTECTED;
 
 /* get autoplay MRL list from input plugin named <plugin_id> */
-char **xine_get_autoplay_mrls (xine_t *self,
-			       const char *plugin_id,
-			       int *num_mrls) XINE_PROTECTED;
+const char * const *xine_get_autoplay_mrls (xine_t *self,
+					    const char *plugin_id,
+					    int *num_mrls) XINE_PROTECTED;
 
 /* get a list of file extensions for file types supported by xine
  * the list is separated by spaces
@@ -1136,12 +1178,6 @@ void xine_plugins_garbage_collector(xine_t *self) XINE_PROTECTED;
  * visual specific gui <-> xine engine communication                 *
  *********************************************************************/
 
-#ifndef XINE_DISABLE_DEPRECATED_FEATURES
-/* talk to video output driver - old method */
-int    xine_gui_send_vo_data (xine_stream_t *self,
-			      int type, void *data) XINE_PROTECTED XINE_DEPRECATED;
-#endif
-
 /* new (preferred) method to talk to video driver. */
 int    xine_port_send_gui_data (xine_video_port_t *vo,
 			        int type, void *data) XINE_PROTECTED;
@@ -1427,7 +1463,7 @@ typedef struct {
 
   HWND      WndHnd;     /* handle of window associated with primary surface */
   HINSTANCE HInst;      /* handle of windows application instance */
-  RECT      WndRect;    /* rect of window client points translated to screen 
+  RECT      WndRect;    /* rect of window client points translated to screen
                          * cooridnates */
   int       FullScreen; /* is window fullscreen */
   HBRUSH    Brush;      /* window brush for background color */
@@ -1491,12 +1527,12 @@ typedef struct {
 #define CHECK_XV        6
 
 struct xine_health_check_s {
-  int         status;
   const char* cdrom_dev;
   const char* dvd_dev;
-  char*       msg;
-  char*       title;
-  char*       explanation;
+  const char* msg;
+  const char* title;
+  const char* explanation;
+  int         status;
 };
 
 typedef struct xine_health_check_s xine_health_check_t;
@@ -1518,7 +1554,7 @@ xine_health_check_t* xine_health_check(xine_health_check_t*, int check_num) XINE
 #define XINE_CONFIG_TYPE_NUM     4
 #define XINE_CONFIG_TYPE_BOOL    5
 
-/* For the string type (1.1.4 and later). These are stored in num_value. */ 
+/* For the string type (1.1.4 and later). These are stored in num_value. */
 #define XINE_CONFIG_STRING_IS_STRING		0
 #define XINE_CONFIG_STRING_IS_FILENAME		1
 #define XINE_CONFIG_STRING_IS_DEVICE_NAME	2
@@ -1533,17 +1569,17 @@ struct xine_cfg_entry_s {
 
   int              type;
 
+  /* user experience level */
+  int              exp_level; /* 0 => beginner,
+			        10 => advanced user,
+			        20 => expert */
+
   /* type unknown */
   char            *unknown_value;
 
   /* type string */
   char            *str_value;
   char            *str_default;
-#ifndef XINE_DISABLE_DEPRECATED_FEATURES
-  char            *str_sticky;
-#else
-  void            *dummy;
-#endif
 
   /* common to range, enum, num, bool;
    * num_value is also used by string to indicate what's required:
@@ -1566,11 +1602,6 @@ struct xine_cfg_entry_s {
   const char      *description;
   const char      *help;
 
-  /* user experience level */
-  int              exp_level; /* 0 => beginner,
-			        10 => advanced user,
-			        20 => expert */
-
   /* callback function and data for live changeable values */
   /* some config entries will take effect immediately, although they
    * do not have a callback registered; such values will have some
@@ -1814,12 +1845,13 @@ void xine_config_reset (xine_t *self) XINE_PROTECTED;
  * xine event struct
  */
 typedef struct {
-  int                              type;   /* event type (constants see above) */
   xine_stream_t                   *stream; /* stream this event belongs to     */
 
   void                            *data;   /* contents depending on type */
   int                              data_length;
 
+  int                              type;   /* event type (constants see above) */
+
   /* you do not have to provide this, it will be filled in by xine_event_send() */
   struct timeval                   tv;     /* timestamp of event creation */
 } xine_event_t;
@@ -1924,8 +1956,8 @@ typedef struct {
  * mrl reference data is sent by demuxers when a reference stream is found.
  * this stream just contains pointers (urls) to the real data, which are
  * passed to frontend using this event type. (examples: .asx, .mov and .ram)
- * 
- * ideally, frontends should add these mrls to a "hierarchical playlist". 
+ *
+ * ideally, frontends should add these mrls to a "hierarchical playlist".
  * that is, instead of the original file, the ones provided here should be
  * played instead. on pratice, just using a simple playlist should work.
  *
@@ -1941,7 +1973,7 @@ typedef struct {
  *   2) http://another/another.avi
  *
  * 1 and 2 are the original items on this playlist. 1a and 1b were received
- * by events (they are the mrl references enclosed in 1). 1a is played after 
+ * by events (they are the mrl references enclosed in 1). 1a is played after
  * receiving the finished event from 1. note: 1b is usually ignored, it should
  * only be used in case 1a fails to open.
  *
@@ -1961,12 +1993,12 @@ typedef struct {
 /*const char          title[]; ** immediately follows MRL's terminating NUL */
 } xine_mrl_reference_data_ext_t;
 
-/* 
+/*
  * configuration options for video4linux-like input plugins
  */
 typedef struct {
   /* input selection */
-  int input;                    /* select active input from card */ 
+  int input;                    /* select active input from card */
   int channel;                  /* channel number */
   int radio;                    /* ask for a radio channel */
   uint32_t frequency;           /* frequency divided by 62.5KHz or 62.5 Hz */
@@ -1983,13 +2015,13 @@ typedef struct {
   int frame_height;             /* scaled frame height */
 
   /* let some spare space so we can add new fields without breaking
-   * binary api compatibility. 
+   * binary api compatibility.
    */
   uint32_t spare[20];
 
   /* used by pvr plugin */
   int32_t session_id;           /* -1 stops pvr recording */
-     
+
 } xine_set_v4l2_data_t;
 
 /*
@@ -2006,7 +2038,7 @@ typedef struct {
   int gop_closure;              /* open/closed GOP */
   int b_frames;                 /* number of B frames to use */
   int aspect_ratio;             /* XINE_VO_ASPECT_xxx */
-  
+
   /* let some spare space so we can add new fields without breaking
    * binary api compatibility.
    */
@@ -2021,7 +2053,7 @@ typedef struct {
 
 #ifdef XINE_ENABLE_EXPERIMENTAL_FEATURES
 
-/* 
+/*
  * ask pvr to save (ie. do not discard) the current session
  * see comments on input_pvr.c to understand how it works.
  */
@@ -2032,7 +2064,7 @@ typedef struct {
    * 1 = save from last sync point
    * 2 = save everything on current session
    */
-  int mode; 
+  int mode;
   int id;
   char name[256]; /* name for saving, might be longer */
 } xine_pvr_save_data_t;
@@ -2042,7 +2074,7 @@ typedef struct {
    * 0 = non realtime
    * 1 = realtime
    */
-  int mode; 
+  int mode;
 } xine_pvr_realtime_t;
 
 typedef struct {
@@ -2073,7 +2105,7 @@ typedef struct {
  * Defined message types for XINE_EVENT_UI_MESSAGE
  * This is the mechanism to report async errors from engine.
  *
- * If frontend knows about the XINE_MSG_xxx type it may safely 
+ * If frontend knows about the XINE_MSG_xxx type it may safely
  * ignore the 'explanation' field and provide its own custom
  * dialogue to the 'parameters'.
  *
@@ -2094,6 +2126,7 @@ typedef struct {
 #define XINE_MSG_AUDIO_OUT_UNAVAILABLE	11 /* none               */
 #define XINE_MSG_PERMISSION_ERROR       12 /* (file name or mrl) */
 #define XINE_MSG_FILE_EMPTY             13 /* file is empty      */
+#define XINE_MSG_AUTHENTICATION_NEEDED	14 /* (mrl, likely http) */
 
 /* opaque xine_event_queue_t */
 typedef struct xine_event_queue_s xine_event_queue_t;
@@ -2167,8 +2200,11 @@ void xine_event_send (xine_stream_t *stream, const xine_event_t *event) XINE_PRO
 /* yellow text, black border, transparent background */
 #define XINE_TEXTPALETTE_YELLOW_BLACK_TRANSPARENT   3
 
-#define XINE_OSD_CAP_FREETYPE2 0x0001 /* freetype2 support compiled in     */
-#define XINE_OSD_CAP_UNSCALED  0x0002 /* unscaled overlays supp. by vo drv */
+#define XINE_OSD_CAP_FREETYPE2     0x0001 /* freetype2 support compiled in     */
+#define XINE_OSD_CAP_UNSCALED      0x0002 /* unscaled overlays supp. by vo drv */
+#define XINE_OSD_CAP_CUSTOM_EXTENT 0x0004 /* hardware scaled to match video output window */
+#define XINE_OSD_CAP_ARGB_LAYER    0x0008 /* supports ARGB true color pixmaps */
+#define XINE_OSD_CAP_VIDEO_WINDOW  0x0010 /* can scale video to an area within osd extent */
 
 typedef struct xine_osd_s xine_osd_t;
 
@@ -2196,7 +2232,7 @@ void        xine_osd_get_text_size (xine_osd_t *self, const char *text,
    as 'fontname' here */
 int         xine_osd_set_font      (xine_osd_t *self, const char *fontname,
 				    int size) XINE_PROTECTED;
-/* 
+/*
  * specifying encoding of texts
  *   ""   ... means current locale encoding (default)
  *   NULL ... means latin1
@@ -2233,6 +2269,34 @@ void        xine_osd_get_palette   (xine_osd_t *self, uint32_t *color,
 void        xine_osd_set_palette   (xine_osd_t *self,
 				    const uint32_t *const color,
 				    const uint8_t *const trans ) XINE_PROTECTED;
+
+/*
+ * Set an ARGB buffer to be blended into video.
+ * The buffer must stay valid while the OSD is on screen.
+ * Pass a NULL pointer to safely remove the buffer from
+ * the OSD layer. Only the dirty area  will be
+ * updated on screen. For convenience the whole
+ * OSD object will be considered dirty when setting
+ * a different buffer pointer.
+ * see also XINE_OSD_CAP_ARGB_LAYER
+ */
+void xine_osd_set_argb_buffer(xine_osd_t *self, uint32_t *argb_buffer,
+                              int dirty_x, int dirty_y, int dirty_width, int dirty_height) XINE_PROTECTED;
+
+/*
+ * define extent of reference coordinate system
+ * for video resolution independent osds.
+ * see also XINE_OSD_CAP_CUSTOM_EXTENT
+ */
+void xine_osd_set_extent(xine_osd_t *self, int extent_width, int extent_height) XINE_PROTECTED;
+
+/*
+ * define area within osd extent to output
+ * video to while osd is on screen
+ * see also XINE_OSD_CAP_VIDEO_WINDOW
+ */
+void xine_osd_set_video_window(xine_osd_t *self, int window_x, int window_y, int window_width, int window_height) XINE_PROTECTED;
+
 /*
  * close osd rendering engine
  * loaded fonts are unloaded
@@ -2240,50 +2304,6 @@ void        xine_osd_set_palette   (xine_osd_t *self,
  */
 void        xine_osd_free          (xine_osd_t *self) XINE_PROTECTED;
 
-
-#ifndef XINE_DISABLE_DEPRECATED_FEATURES
-
-/*********************************************************************
- * TV-mode API, to make it possible to use nvtvd to view movies      *
- *********************************************************************/
-
-/* These functions are just dummies to maintain API compatibility.
- * You should use libnvtvsimple in your frontend instead. */
-
-typedef enum {
-	XINE_TVSYSTEM_PAL = 0,
-	XINE_TVSYSTEM_NTSC
-} xine_tvsystem XINE_DEPRECATED;
-
-/* connect to nvtvd server and save current TV and X settings */
-int xine_tvmode_init (xine_t *self) XINE_PROTECTED XINE_DEPRECATED;
-
-/* Turn tvmode on/off (1/0)*/
-int xine_tvmode_use(xine_t *self, int use_tvmode) XINE_PROTECTED XINE_DEPRECATED;
-
-/* Set which tv system to use: XINE_TVSYSTEM_PAL or XINE_TVSYSTEM_NTSC */
-void xine_tvmode_set_tvsystem(xine_t *self, xine_tvsystem system) XINE_PROTECTED XINE_DEPRECATED;
-
-/* try to change TV state if enabled
- *   type select 'regular' (0) or 'TV' (1) state
- *   width frame width the mode should match best or 0 if unknown
- *   height frame height the mode should match best or 0 if unknown
- *   fps frame rate the mode should match best or 0 if unknown
- * returns: finally selected state
- */
-int xine_tvmode_switch (xine_t *self, int type, int width, int height, double fps) XINE_PROTECTED XINE_DEPRECATED;
-
-/* adapt (maximum) output size to visible area if necessary and return pixel
- * aspect and real frame rate if available
- */
-void xine_tvmode_size (xine_t *self, int *width, int *height,
-		       double *pixelratio, double *fps) XINE_PROTECTED XINE_DEPRECATED;
-
-/* restore old TV and X settings and close nvtvd connection */
-void xine_tvmode_exit (xine_t *self) XINE_PROTECTED XINE_DEPRECATED;
-
-#endif
-
 #ifdef __cplusplus
 }
 #endif
