@@ -38,14 +38,12 @@ extern "C" {
 
 #include <pthread.h>
 
-#ifdef XINE_COMPILE
-#  include "xine.h"
-#  include "buffer.h"
-#else
-#  include <xine.h>
-#  include <xine/buffer.h>
-#endif
+#include <xine.h>
+#include <xine/buffer.h>
 
+#ifdef XINE_COMPILE
+#  include <xine/plugin_catalog.h>
+#endif
 
 typedef struct vo_frame_s vo_frame_t;
 typedef struct vo_driver_s vo_driver_t;
@@ -64,6 +62,14 @@ struct vo_frame_s {
    * member functions
    */
 
+  /* Provide a copy of the frame's image in an image format already known to xine. data's member */
+  /* have already been intialized to frame's content on entry, so it's usually only necessary to */
+  /* change format and img_size. In case img is set, it will point to a memory block of suitable */
+  /* size (size has been determined by a previous call with img == NULL). img content and img_size */
+  /* must adhere to the specification of _x_get_current_frame_data(). */
+  /* Currently this is needed for all image formats except XINE_IMGFMT_YV12 and XINE_IMGFMT_YUY2. */
+  void (*proc_provide_standard_frame_data) (vo_frame_t *vo_img, xine_current_frame_data_t *data);
+
   /* Duplicate picture data and acceleration specific data of a frame. */
   /* if the image format isn't already known by Xine. Currently this is needed */
   /* For all image formats except XINE_IMGFMT_YV12 and XINE_IMGFMT_YUY2 */
@@ -123,6 +129,9 @@ struct vo_frame_s {
   /* cropping to be done */
   int                        crop_left, crop_right, crop_top, crop_bottom;
 
+  int                        lock_counter;
+  pthread_mutex_t            mutex; /* protect access to lock_count */
+
   /* extra info coming from input or demuxers */
   extra_info_t              *extra_info;
 
@@ -146,13 +155,14 @@ struct vo_frame_s {
   /* displacement for overlays */
   int                       overlay_offset_x, overlay_offset_y;
 
+  /* pointer to the next frame in display order, used by some vo deint */
+  struct vo_frame_s         *future_frame;
+
   /*
    * that part is used only by video_out.c for frame management
    * obs: changing anything here will require recompiling vo drivers
    */
   struct vo_frame_s         *next;
-  int                        lock_counter;
-  pthread_mutex_t            mutex; /* protect access to lock_count */
 
   int                        id; /* debugging - track this frame */
   int                        is_first;
@@ -186,6 +196,9 @@ struct xine_video_port_s {
 			    uint32_t height, double ratio,
 			    int format, int flags);
 
+  /* create a new grab video frame */
+  xine_grab_video_frame_t* (*new_grab_video_frame) (xine_video_port_t *self);
+
   /* retrieves the last displayed frame (useful for taking snapshots) */
   vo_frame_t* (*get_last_frame) (xine_video_port_t *self);
 
@@ -198,6 +211,9 @@ struct xine_video_port_s {
   /* flush video_out fifo */
   void (*flush) (xine_video_port_t *self);
 
+  /* trigger immediate drawing */
+  void (*trigger_drawing) (xine_video_port_t *self);
+
   /* Get/Set video property
    *
    * See VO_PROP_* bellow
@@ -235,6 +251,7 @@ struct xine_video_port_s {
 #define VO_PROP_PAN_SCAN              9
 #define VO_PROP_TVMODE		      10
 #define VO_PROP_MAX_NUM_FRAMES        11
+#define VO_PROP_GAMMA		      12
 #define VO_PROP_ZOOM_Y                13
 #define VO_PROP_DISCARD_FRAMES        14 /* not used by drivers */
 #define VO_PROP_WINDOW_WIDTH          15 /* read-only */
@@ -245,7 +262,13 @@ struct xine_video_port_s {
 #define VO_PROP_OUTPUT_HEIGHT         20 /* read-only */
 #define VO_PROP_OUTPUT_XOFFSET        21 /* read-only */
 #define VO_PROP_OUTPUT_YOFFSET        22 /* read-only */
-#define VO_NUM_PROPERTIES             23
+#define VO_PROP_SHARPNESS             24
+#define VO_PROP_NOISE_REDUCTION       25
+#define VO_PROP_BUFS_TOTAL            26 /* read-only */
+#define VO_PROP_BUFS_FREE             27 /* read-only */
+#define VO_PROP_MAX_VIDEO_WIDTH       28 /* read-only */
+#define VO_PROP_MAX_VIDEO_HEIGHT      29 /* read-only */
+#define VO_NUM_PROPERTIES             30
 
 /* number of colors in the overlay palette. Currently limited to 256
    at most, because some alphablend functions use an 8-bit index into
@@ -267,6 +290,13 @@ struct xine_video_port_s {
 #define VO_PAN_SCAN_FLAG     4
 #define VO_INTERLACED_FLAG   8
 #define VO_NEW_SEQUENCE_FLAG 16 /* set after MPEG2 Sequence Header Code (used by XvMC) */
+#define VO_CHROMA_422        32 /* used by VDPAU, default is chroma_420 */
+#define VO_STILL_IMAGE       64
+
+/* ((mpeg_color_matrix << 1) | color_range) inside frame.flags bits 11-8 */
+#define VO_FULLRANGE 0x100
+#define VO_GET_FLAGS_CM(flags) ((flags >> 8) & 15)
+#define VO_SET_FLAGS_CM(cm,flags) flags = ((flags) & ~0xf00) | (((cm) & 15) << 8)
 
 /* video driver capabilities */
 #define VO_CAP_YV12                   0x00000001 /* driver can handle YUV 4:2:0 pictures */
@@ -276,6 +306,27 @@ struct xine_video_port_s {
 #define VO_CAP_UNSCALED_OVERLAY       0x00000010 /* driver can blend overlay at output resolution */
 #define VO_CAP_CROP                   0x00000020 /* driver can crop */
 #define VO_CAP_XXMC                   0x00000040 /* driver can use extended XvMC */
+#define VO_CAP_VDPAU_H264             0x00000080 /* driver can use VDPAU for H264 */
+#define VO_CAP_VDPAU_MPEG12           0x00000100 /* driver can use VDPAU for mpeg1/2 */
+#define VO_CAP_VDPAU_VC1              0x00000200 /* driver can use VDPAU for VC1 */
+#define VO_CAP_VDPAU_MPEG4            0x00000400 /* driver can use VDPAU for mpeg4-part2 */
+#define VO_CAP_VAAPI                  0x00000800 /* driver can use VAAPI */
+#define VO_CAP_COLOR_MATRIX           0x00004000 /* driver can use alternative yuv->rgb matrices */
+#define VO_CAP_FULLRANGE              0x00008000 /* driver handles fullrange yuv */
+#define VO_CAP_HUE                    0x00010000
+#define VO_CAP_SATURATION             0x00020000
+#define VO_CAP_CONTRAST               0x00040000
+#define VO_CAP_BRIGHTNESS             0x00080000
+#define VO_CAP_COLORKEY               0x00100000
+#define VO_CAP_AUTOPAINT_COLORKEY     0x00200000
+#define VO_CAP_ZOOM_X                 0x00400000
+#define VO_CAP_ZOOM_Y                 0x00800000
+#define VO_CAP_CUSTOM_EXTENT_OVERLAY  0x01000000 /* driver can blend custom extent overlay to output extent */
+#define VO_CAP_ARGB_LAYER_OVERLAY     0x02000000 /* driver supports true color overlay */
+#define VO_CAP_VIDEO_WINDOW_OVERLAY   0x04000000 /* driver can scale video to an area within overlay */
+#define VO_CAP_GAMMA                  0x08000000
+#define VO_CAP_SHARPNESS              0x10000000
+#define VO_CAP_NOISE_REDUCTION        0x20000000
 
 
 /*
@@ -287,7 +338,7 @@ struct xine_video_port_s {
  * from generic vo functions.
  */
 
-#define VIDEO_OUT_DRIVER_IFACE_VERSION  21
+#define VIDEO_OUT_DRIVER_IFACE_VERSION  22
 
 struct vo_driver_s {
 
@@ -350,12 +401,25 @@ struct vo_driver_s {
    */
   int (*redraw_needed) (vo_driver_t *self);
 
+  /* Create a new grab video frame */
+  xine_grab_video_frame_t* (*new_grab_video_frame)(vo_driver_t *self);
+
   /*
    * free all resources, close driver
    */
   void (*dispose) (vo_driver_t *self);
 
-  void *node; /* needed by plugin_loader */
+  /**
+   * @brief Pointer to the loaded plugin node.
+   *
+   * Used by the plugins loader. It's an opaque type when using the
+   * structure outside of xine's build.
+   */
+#ifdef XINE_COMPILE
+  plugin_node_t *node;
+#else
+  void *node;
+#endif
 };
 
 struct video_driver_class_s {
@@ -365,16 +429,22 @@ struct video_driver_class_s {
    */
   vo_driver_t* (*open_plugin) (video_driver_class_t *self, const void *visual);
 
-  /*
-   * return short, human readable identifier for this plugin class
+  /**
+   * @brief short human readable identifier for this plugin class
    */
-  char* (*get_identifier) (video_driver_class_t *self);
+  const char *identifier;
 
-  /*
-   * return human readable (verbose = 1 line) description for
-   * this plugin class
+  /**
+   * @brief human readable (verbose = 1 line) description for this plugin class
+   *
+   * The description is passed to gettext() to internationalise.
    */
-  char* (*get_description) (video_driver_class_t *self);
+  const char *description;
+
+  /**
+   * @brief Optional non-standard catalog to use with dgettext() for description.
+   */
+  const char *text_domain;
 
   /*
    * free all class-related resources
@@ -382,12 +452,22 @@ struct video_driver_class_s {
   void (*dispose) (video_driver_class_t *self);
 };
 
+#define default_video_driver_class_dispose (void (*) (video_driver_class_t *this))free
 
 typedef struct rle_elem_s {
   uint16_t len;
   uint16_t color;
 } rle_elem_t;
 
+typedef struct argb_layer_s {
+  pthread_mutex_t  mutex;
+  uint32_t        *buffer;
+  /* dirty area */
+  int x1, y1;
+  int x2, y2;
+  int ref_count;
+} argb_layer_t;
+
 struct vo_overlay_s {
 
   rle_elem_t       *rle;           /* rle code buffer                  */
@@ -398,6 +478,16 @@ struct vo_overlay_s {
   int               width;         /* width of subpicture area         */
   int               height;        /* height of subpicture area        */
 
+  /* area within osd extent to scale video to */
+  int               video_window_x;
+  int               video_window_y;
+  int               video_window_width;
+  int               video_window_height;
+
+  /* extent of reference coordinate system */
+  int               extent_width;
+  int               extent_height;
+
   uint32_t          color[OVL_PALETTE_SIZE];  /* color lookup table     */
   uint8_t           trans[OVL_PALETTE_SIZE];  /* mixer key table        */
   int               rgb_clut;      /* true if clut was converted to rgb */
@@ -412,8 +502,11 @@ struct vo_overlay_s {
   int               hili_rgb_clut; /* true if clut was converted to rgb */
 
   int               unscaled;      /* true if it should be blended unscaled */
+
+  argb_layer_t     *argb_layer;
 };
 
+void set_argb_layer_ptr(argb_layer_t **dst, argb_layer_t *src);
 
 /* API to video_overlay manager
  *
@@ -440,11 +533,12 @@ struct video_overlay_manager_s {
                                   vo_driver_t *output, vo_frame_t *vo_img, int enabled);
 };
 
-/*
- * build a video_out_port from
- * a given video driver
+/**
+ * @brief Build a video output port from a given video driver.
+ *
+ * @internal
  */
-xine_video_port_t *_x_vo_new_port (xine_t *xine, vo_driver_t *driver, int grabonly) XINE_MALLOC XINE_PROTECTED;
+xine_video_port_t *_x_vo_new_port (xine_t *xine, vo_driver_t *driver, int grabonly) XINE_MALLOC;
 
 #ifdef __cplusplus
 }
